/////////////////////////////////////////////////////////////////////////
// This program is free software; you can redistribute it              //
// and/or modify it under the terms of the GNU General Public License  //
// version 2 (or higher) as published by the Free Software Foundation. //
//                                                                     //
// This program is distributed in the hope that it will be useful, but //
// WITHOUT ANY WARRANTY; without even the implied warranty of          //
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU   //
// General Public License for more details.                            //
//                                                                     //
// Written and (C) by                                                  //
// Engin Tola                                                          //
//                                                                     //
// web   : http://cvlab.epfl.ch/~tola                                  //
// email : engin.tola@epfl.ch                                          //
//                                                                     //
// If you use this code for research purposes, please refer to the     //
// webpage above                                                       //
/////////////////////////////////////////////////////////////////////////

#ifndef KUTILITY_IMAGE_IO_PNM_H
#define KUTILITY_IMAGE_IO_PNM_H

#include <fstream>
#include "string.h"
#include <cstdlib>
#include "limits.h"

#ifndef uchar
typedef unsigned char uchar;
#endif

namespace kutility
{
   void load_pbm(const char* name, uchar* &data, int &height, int &width);
   void load_pgm(const char* name, uchar* &data, int &height, int &width);
   void load_ppm(const char* name, uchar* &data, int &height, int &width);

   void save_pbm(const char* name, uchar *im, int height, int width);

   template<class T>
   void save_pgm(const char* name, T *im, int height, int width)
   {
      std::ofstream file(name, std::ios::out | std::ios::binary);

      file << "P5\n" << width << " " << height << "\n" << UCHAR_MAX << "\n";

      for( int k=0; k<width*height; k++ )
      {
         file <<(uchar)im[k];
      }
      file.close();
   }

   template<class T>
   void save_ppm(const char* name, T *im, int height, int width)
   {
      std::ofstream file(name, std::ios::out | std::ios::binary);

      file << "P6\n" << width << " " << height << "\n" << UCHAR_MAX << "\n";
      for( int k=0; k<3*width*height; k++ )
      {
         file <<(uchar)im[k];
      }
      file.close();
   }

   void get_size_ppm(const char* name, int &height, int &width);
}

#endif
